@@ -7,15 +7,16 @@ import time
 import traceback
 import uuid
 from os import environ as env
-from typing import List, Optional
+from typing import List
 from urllib.request import urlopen
+import shortuuid
 
 from dotenv import find_dotenv, load_dotenv
-from fastapi import Header, HTTPException, Request
+from fastapi import Header, HTTPException
 from jose import jwt
 from pydantic import BaseModel  # pylint: disable=no-name-in-module
 
-from imjoy.core import UserInfo, VisibilityEnum, TokenConfig, all_users, all_workspaces
+from imjoy.core import UserInfo, TokenConfig
 
 logging.basicConfig(stream=sys.stdout)
 logger = logging.getLogger("imjoy-core")
@@ -59,14 +60,23 @@ class ValidToken(BaseModel):
         )
 
 
-def login_required(request: Request, authorization: str = Header(None)):
-    """Return token if login is ok."""
-    return valid_token(authorization, request)
+def login_optional(authorization: str = Header(None)):
+    """Return user info or create an anonymouse user.
+
+    If authorization code is valid the user info is returned,
+    If the code is invalid an an anonymouse user is created.
+    """
+    return parse_token(authorization, allow_anonymouse=True)
 
 
-def admin_required(request: Request, authorization: str = Header(None)):
-    """Return token if the token has an admin role."""
-    token = valid_token(authorization, request)
+def login_required(authorization: str = Header(None)):
+    """Return user info if authorization code is valid."""
+    return parse_token(authorization)
+
+
+def admin_required(authorization: str = Header(None)):
+    """Return user info if the authorization code has an admin role."""
+    token = parse_token(authorization)
     roles = token.credentials.get("https://api.imjoy.io/roles", [])
     if "admin" not in roles:
         raise HTTPException(status_code=401, detail="Admin required")
@@ -93,11 +103,17 @@ def get_user_id(token):
 
 def get_user_info(token):
     """Return the user info from the token."""
-    return {
-        "user_id": token.credentials.get("sub"),
-        "email": token.credentials.get("https://api.imjoy.io/email"),
-        "roles": token.credentials.get("https://api.imjoy.io/roles", []),
-    }
+    credentials = token.credentials
+    expires_at = credentials["exp"]
+    return UserInfo(
+        id=credentials.get("sub"),
+        is_anonymous=False,
+        email=credentials.get("https://api.imjoy.io/email"),
+        parent=credentials.get("parent", None),
+        roles=credentials.get("https://api.imjoy.io/roles", []),
+        scopes=token.scopes,
+        expires_at=expires_at,
+    )
 
 
 JWKS = None
@@ -141,7 +157,7 @@ def simulate_user_token(returned_token, request):
         ].split(",")
 
 
-def valid_token(authorization: str, request: Optional[Request] = None):
+def valid_token(authorization: str):
     """Validate token."""
     if not authorization:
         raise HTTPException(status_code=401, detail="Authorization header is expected")
@@ -150,7 +166,7 @@ def valid_token(authorization: str, request: Optional[Request] = None):
 
     if parts[0].lower() != "bearer":
         raise HTTPException(
-            status_code=401, detail="Authorization header must start with" " Bearer"
+            status_code=401, detail="Authorization header must start with Bearer"
         )
     if len(parts) == 1:
         raise HTTPException(status_code=401, detail="Token not found")
@@ -182,19 +198,6 @@ def valid_token(authorization: str, request: Optional[Request] = None):
             credentials=payload, scopes=payload["scope"].split(" ")
         )
 
-        # This is needed for patching the test token
-        if "create:roles" in payload["scope"]:
-            if "https://api.imjoy.io/roles" not in returned_token.credentials:
-                returned_token.credentials["https://api.imjoy.io/roles"] = ["admin"]
-            if "https://api.imjoy.io/email" not in returned_token.credentials:
-                returned_token.credentials["https://api.imjoy.io/email"] = None
-
-        if (
-            "admin" in returned_token.credentials["https://api.imjoy.io/roles"]
-            and request
-        ):
-            simulate_user_token(returned_token, request)
-
         return returned_token
 
     except jwt.ExpiredSignatureError as err:
@@ -205,23 +208,66 @@ def valid_token(authorization: str, request: Optional[Request] = None):
         raise HTTPException(status_code=401, detail=traceback.format_exc()) from err
 
 
-def parse_token(authorization):
-    """Parse the token."""
-    parts = authorization.split()
-    if parts[0].lower() != "bearer":
-        raise Exception("Authorization header must start with" " Bearer")
-    if len(parts) == 1:
-        raise Exception("Token not found")
-    if len(parts) > 2:
-        raise Exception("Authorization header must be 'Bearer' token")
+def generate_anonymouse_user():
+    """Generate user info for a anonymouse user."""
+    iat = time.time()
+    return ValidToken(
+        credentials={
+            "iss": "https://imjoy.eu.auth0.com/",
+            "sub": shortuuid.uuid(),  # user_id
+            "aud": "https://imjoy.eu.auth0.com/api/v2/",
+            "iat": iat,
+            "exp": iat + 600,
+            "azp": "aormkFV0l7T0shrIwjdeQIUmNLt09DmA",
+            "scope": "",
+            "gty": "client-credentials",
+            "https://api.imjoy.io/roles": [],
+            "https://api.imjoy.io/email": "anonymous@imjoy.io",
+        },
+        scopes=[],
+    )
 
-    token = parts[1]
-    if not token.startswith("imjoy@"):
+
+def parse_token(authorization: str, allow_anonymouse=False):
+    """Parse the token."""
+    if not authorization:
+        if allow_anonymouse:
+            info = generate_anonymouse_user()
+            return get_user_info(info)
+        raise HTTPException(status_code=401, detail="Authorization header is expected")
+
+    if authorization.startswith("Bearer ") or authorization.startswith("bearer "):
+        parts = authorization.split()
+        if parts[0].lower() != "bearer":
+            raise HTTPException(
+                status_code=401, detail="Authorization header must start with" " Bearer"
+            )
+        if len(parts) == 1:
+            raise HTTPException(status_code=401, detail="Token not found")
+        if len(parts) > 2:
+            raise HTTPException(
+                status_code=401, detail="Authorization header must be 'Bearer' token"
+            )
+
+        token = parts[1]
+    else:
+        token = authorization
+
+    if "@imjoy@" not in token:
         # auth0 token
-        return get_user_info(valid_token(authorization))
-    # generated token
-    token = token.lstrip("imjoy@")
-    return jwt.decode(token, JWT_SECRET, algorithms=["HS256"])
+        info = valid_token(authorization)
+    else:
+        # generated token
+        token = token.split("@imjoy@")[1]
+        payload = jwt.decode(
+            token,
+            JWT_SECRET,
+            algorithms=["HS256"],
+            audience=AUTH0_AUDIENCE,
+            issuer="https://imjoy.io/",
+        )
+        info = ValidToken(credentials=payload, scopes=payload["scope"].split(" "))
+    return get_user_info(info)
 
 
 def generate_presigned_token(user_info: UserInfo, config: TokenConfig):
@@ -231,72 +277,28 @@ def generate_presigned_token(user_info: UserInfo, config: TokenConfig):
     Child user may generate more child user token if it has admin permission.
     """
     scopes = config.scopes
-    for scope in scopes:
-        if not check_permission(scope, user_info):
-            raise PermissionError(f"User has no permission to scope: {scope}")
 
     # always generate a new user id
-    uid = str(uuid.uuid4())
-    expires_in = config.expires_in
-    if expires_in:
-        expires_at = time.time() + expires_in
-    else:
-        expires_at = None
+    uid = shortuuid.uuid()
+    expires_in = config.expires_in or 10800
+    expires_at = time.time() + expires_in
+
+    parent = user_info.parent if user_info.parent else user_info.id
+
     token = jwt.encode(
         {
-            "scopes": scopes,
-            "expires_at": expires_at,
-            "user_id": uid,
-            "parent": user_info.parent if user_info.parent else user_info.id,
-            "email": config.email,
-            "roles": [],
+            "iss": "https://imjoy.io/",
+            "sub": uid,  # user_id
+            "aud": "https://imjoy.eu.auth0.com/api/v2/",
+            "iat": expires_in,
+            "exp": expires_at,
+            "scope": " ".join(scopes),
+            "parent": parent,
+            "gty": "client-credentials",
+            "https://api.imjoy.io/roles": [],
+            "https://api.imjoy.io/email": config.email,
         },
         JWT_SECRET,
         algorithm="HS256",
     )
-    return {"token": "imjoy@" + token, "id": uid}
-
-
-def check_permission(workspace, user_info):
-    """Check user permission for a workspace."""
-    # pylint: disable=too-many-return-statements
-    if isinstance(workspace, str):
-        workspace = all_workspaces.get(workspace)
-        if not workspace:
-            logger.warning("Workspace %s not found", workspace)
-            return False
-
-    if workspace.name == user_info.id:
-        return True
-
-    if user_info.parent:
-        parent = all_users.get(user_info.parent)
-        if not parent:
-            return False
-        if not check_permission(workspace, parent):
-            return False
-        # if the parent has access
-        # and the workspace is in the scopes
-        # then we allow the access
-        if workspace.name in user_info.scopes:
-            return True
-
-    _id = user_info.email or user_info.id
-
-    if _id in workspace.owners:
-        return True
-
-    if workspace.visibility == VisibilityEnum.public:
-        if workspace.deny_list and user_info.email not in workspace.deny_list:
-            return True
-    elif workspace.visibility == VisibilityEnum.protected:
-        if workspace.allow_list and user_info.email in workspace.allow_list:
-            return True
-
-    if "admin" in user_info.roles:
-        logger.info(
-            "Allowing access to %s for admin user %s", workspace.name, user_info.id
-        )
-        return True
-
-    return False
+    return uid + "@imjoy@" + token
