@@ -3,10 +3,13 @@ import copy
 import json
 import os
 import string
+import secrets
 import sys
 import threading
 import time
 import uuid
+import posixpath
+from typing import List, Optional
 from importlib import import_module
 
 if sys.platform == "win32":
@@ -26,6 +29,51 @@ if sys.platform == "win32":
 _SERVER_THREAD = None
 
 
+_os_alt_seps: List[str] = list(
+    sep for sep in [os.path.sep, os.path.altsep] if sep is not None and sep != "/"
+)
+
+
+def generate_password(length=20):
+    """Generate a password."""
+    alphabet = string.ascii_letters + string.digits
+    return "".join(secrets.choice(alphabet) for i in range(length))
+
+
+def safe_join(directory: str, *pathnames: str) -> Optional[str]:
+    """Safely join zero or more untrusted path components to a base directory.
+
+    This avoids escaping the base directory.
+    :param directory: The trusted base directory.
+    :param pathnames: The untrusted path components relative to the
+        base directory.
+    :return: A safe path, otherwise ``None``.
+
+    This function is copied from:
+    https://github.com/pallets/werkzeug/blob/fb7ddd89ae3072e4f4002701a643eb247a402b64/src/werkzeug/security.py#L222
+    """
+    parts = [directory]
+
+    for filename in pathnames:
+        if filename != "":
+            filename = posixpath.normpath(filename)
+
+        if (
+            any(sep in filename for sep in _os_alt_seps)
+            or os.path.isabs(filename)
+            or filename == ".."
+            or filename.startswith("../")
+        ):
+            raise Exception(
+                f"Illegal file path: `{filename}`, "
+                "you can only operate within the work directory."
+            )
+
+        parts.append(filename)
+
+    return posixpath.join(*parts)
+
+
 def _show_elfinder_colab(root_dir="/content", port=8765, height=600, width="100%"):
     # pylint: disable=import-error, import-outside-toplevel, no-name-in-module
     from google.colab import output
@@ -37,7 +85,7 @@ def _show_elfinder_colab(root_dir="/content", port=8765, height=600, width="100%
         def start_elfinder():
             global _SERVER_THREAD  # pylint: disable=global-statement
             try:
-                main(["--root-dir={}".format(root_dir), "--port={}".format(port)])
+                main([f"--root-dir={root_dir}", f"--port={port}"])
             except OSError:
                 print("ImJoy-elFinder server already started.")
             _SERVER_THREAD = thread
@@ -53,7 +101,8 @@ def _show_elfinder_colab(root_dir="/content", port=8765, height=600, width="100%
 def _show_elfinder_jupyter(url="/elfinder", height=600, width="100%"):
     from IPython import display  # pylint: disable=import-outside-toplevel
 
-    code = """(async (url, width, height, element) => {
+    code = (
+        """(async (url, width, height, element) => {
         element.appendChild(document.createTextNode(''));
         const iframe = document.createElement('iframe');
         iframe.src = url;
@@ -61,8 +110,8 @@ def _show_elfinder_jupyter(url="/elfinder", height=600, width="100%"):
         iframe.width = width;
         iframe.style.border = 0;
         element.appendChild(iframe);
-        })""" + "({url}, {width}, {height}, element[0])".format(
-        url=json.dumps(url), width=json.dumps(width), height=json.dumps(height)
+        })"""
+        + f"({json.dumps(url)}, {json.dumps(width)}, {json.dumps(height)}, element[0])"
     )
     display.display(display.Javascript(code))
 
@@ -88,11 +137,11 @@ def read_or_generate_token(token_path=None):
     token_path = token_path or os.path.join(os.path.expanduser("~"), ".jupyter_token")
     # read token from file if exists
     try:
-        with open(token_path, "r") as fil:
+        with open(token_path, "r", encoding="utf-8") as fil:
             token = fil.read()
     except FileNotFoundError:
         token = str(uuid.uuid4())
-        with open(token_path, "w") as fil:
+        with open(token_path, "w", encoding="utf-8") as fil:
             fil.write(token)
 
     return token
@@ -101,7 +150,7 @@ def read_or_generate_token(token_path=None):
 def write_token(token, token_path=None):
     """Write token."""
     token_path = token_path or os.path.join(os.path.expanduser("~"), ".jupyter_token")
-    with open(token_path, "w") as fil:
+    with open(token_path, "w", encoding="utf-8") as fil:
         fil.write(token)
 
 
